{*******************************************************************************
 Project       : "Space Shooter ..." ?
 Date          : 2007-07-11
 Version       : 1.3
 Compiler Ver. : Delphi 2007
 Author        : Denny Koberling
 E-Mail        : JasonVoorhees2k@arcor.de
 Copyright     : Copyright (c) 2007 Denny Koberling
*******************************************************************************}

unit UAnimBmp;

interface

uses Windows, SysUtils, Classes, Types, Graphics;

type
  PPixel = ^TPixel;
  TPixel = packed record
    Color: Word; // 16bit RGB565
    Alpha: Byte; //  8bit
    Mask : Byte; //  8bit
  end;

type
  THeader = packed record
    Width    : Integer;
    Height   : Integer;
    AnimCount: Byte;
  end;

  TSubHeader = packed record
    FrameCount: Byte;
    FPS       : Byte;
  end;

type
  TAnimBmp = class(TObject)
  private
    fName          : String;
    fHeader        : THeader;
    fSubHeader     : array of TSubHeader;
    fMap           : array of PPixel;
    fBytesPerLine  : Integer;
    fBytesPerBitmap: Integer;
    function GetFrameCount(aAnim: Byte): Byte;
    function GetFPS(aAnim: Byte): Byte;
    function GetScanLine(aAnim, aFrame: Byte; aRow: Integer): PPixel;
    function GetPixel(aAnim, aFrame: Byte; aX, aY: Integer): PPixel;
  public
    constructor Create(aObjFileName: String);
    destructor Destroy; override;
    property Name: String read fName;
    property Width: Integer read fHeader.Width;
    property Height: Integer read fHeader.Height;
    property AnimCount: Byte read fHeader.AnimCount;
    property FrameCount[aAnim: Byte]: Byte read GetFrameCount;
    property FPS[aAnim: Byte]: Byte read GetFPS;
    property ScanLine[aAnim, aFrame: Byte; aRow: Integer]: PPixel read GetScanLine;
    property Pixel[aAnim, aFrame: Byte; aX, aY: Integer]: PPixel read GetPixel;
  end;

implementation

{ TAnimBmp }

constructor TAnimBmp.Create(aObjFileName: String);
var
  f: TFileStream;
  i: Integer;
begin
  inherited Create;
  fName:=ChangeFileExt(ExtractFileName(aObjFileName), '');
  f:=TFileStream.Create(aObjFileName, fmOpenRead);
  try
    f.ReadBuffer(fHeader, SizeOf(THeader));
    fBytesPerLine:=fHeader.Width*SizeOf(TPixel);
    fBytesPerBitmap:=fBytesPerLine*fHeader.Height;
    SetLength(fSubHeader, fHeader.AnimCount);
    SetLength(fMap, fHeader.AnimCount);
    for i:=0 to fHeader.AnimCount-1 do begin
      f.ReadBuffer(fSubHeader[i], SizeOf(TSubHeader));
      GetMem(fMap[i], fSubHeader[i].FrameCount*fBytesPerBitmap);
      f.ReadBuffer(fMap[i]^, fSubHeader[i].FrameCount*fBytesPerBitmap);
    end;
  finally
    FreeAndNil(f);
  end;
end;

destructor TAnimBmp.Destroy;
var
  i: Integer;
begin
  for i:=0 to fHeader.AnimCount-1 do
    FreeMem(fMap[i], fSubHeader[i].FrameCount*fBytesPerBitmap);
  inherited;
end;

function TAnimBmp.GetFrameCount(aAnim: Byte): Byte;
begin
  Result:=fSubHeader[aAnim].FrameCount;
end;

function TAnimBmp.GetFPS(aAnim: Byte): Byte;
begin
  Result:=fSubHeader[aAnim].FPS;
end;

function TAnimBmp.GetScanLine(aAnim, aFrame: Byte; aRow: Integer): PPixel;
begin
  Result:=PPixel(Integer(fMap[aAnim])+aFrame*fBytesPerBitmap+aRow*fBytesPerLine);
end;

function TAnimBmp.GetPixel(aAnim, aFrame: Byte; aX, aY: Integer): PPixel;
begin
  Result:=PPixel(Integer(fMap[aAnim])+aFrame*fBytesPerBitmap+aY*fBytesPerLine+aX*SizeOf(TPixel));
end;

end.
