{*******************************************************************************
 Project       : "Space Shooter ..." ?
 Date          : 2007-07-11
 Version       : 1.3
 Compiler Ver. : Delphi 2007
 Author        : Denny Koberling
 E-Mail        : JasonVoorhees2k@arcor.de
 Copyright     : Copyright (c) 2007 Denny Koberling
*******************************************************************************}

unit UEnemy;

interface

uses Math, USprite, UType;

type
  TEnemy = class(TSprite);

  TEnemy1 = class(TEnemy)
  public
    constructor Create(aParent: TSprite; aX, aY, aSpeed: Single);
    procedure Move(aMoveCount: Single; aKeys: TKeys; aMouse: TMouse); override;
    procedure Hit(aSprite: TSprite); override;
    function AllowHit(aSprite: TSprite): Boolean; override;
  end;

  TEnemy2 = class(TEnemy)
  public
    constructor Create(aParent: TSprite; aX, aY, aSpeed: Single);
    procedure Move(aMoveCount: Single; aKeys: TKeys; aMouse: TMouse); override;
    procedure Hit(aSprite: TSprite); override;
    function AllowHit(aSprite: TSprite): Boolean; override;
  end;

  TEnemy3 = class(TEnemy)
  public
    constructor Create(aParent: TSprite; aX, aY, aSpeed: Single);
    procedure Move(aMoveCount: Single; aKeys: TKeys; aMouse: TMouse); override;
    procedure Hit(aSprite: TSprite); override;
    function AllowHit(aSprite: TSprite): Boolean; override;
  end;

  TEnemy4 = class(TEnemy)
  public
    constructor Create(aParent: TSprite; aX, aY, aSpeed: Single);
    procedure Move(aMoveCount: Single; aKeys: TKeys; aMouse: TMouse); override;
    procedure Hit(aSprite: TSprite); override;
    function AllowHit(aSprite: TSprite): Boolean; override;
  end;

implementation

uses UTimer, UEngine, UPlayer, UEffect, UWeapon;

{ TEnemy1 }

constructor TEnemy1.Create(aParent: TSprite; aX, aY, aSpeed: Single);
begin
  inherited Create(aParent, 'Enemy1', 4, aX, aY, aSpeed);
  AnimDefaultNr:=1;
  FireTimer:=TTimer.Create(300);
end;

procedure TEnemy1.Move(aMoveCount: Single; aKeys: TKeys; aMouse: TMouse);
begin
  inherited;
  X:=X+Speed*aMoveCount;
  Y:=Sin(DegToRad(X*2))*100+150;

  if X>SCREEN_WIDTH then
    Dead;

  if FireTimer.Fire(aMoveCount) then begin
    Engine.SpriteList.AddSprite(TEnemyWeapon1.Create(Self, X+10, Y+65, 0.6));
    Engine.SpriteList.AddSprite(TEnemyWeapon1.Create(Self, X+85, Y+65, 0.6));
    AnimNr:=3;
  end;
end;

procedure TEnemy1.Hit(aSprite: TSprite);
begin
  Engine.SpriteList.AddSprite(TBang.Create(Self, CenterX, CenterY));
  Dead;
end;

function TEnemy1.AllowHit(aSprite: TSprite): Boolean;
begin
  Result:=(aSprite is TPlayerWeapon) or (aSprite is TPlayer);
end;

{ TEnemy2 }

constructor TEnemy2.Create(aParent: TSprite; aX, aY, aSpeed: Single);
begin
  inherited Create(aParent, 'Enemy1', 4, aX, aY, aSpeed);
  AnimDefaultNr:=1;
  FireTimer:=TTimer.Create(300);
end;

procedure TEnemy2.Move(aMoveCount: Single; aKeys: TKeys; aMouse: TMouse);
begin
  inherited;
  Y:=Y+Speed*aMoveCount;
  if Y>SCREEN_HEIGHT then
    Dead;

  if FireTimer.Fire(aMoveCount) then begin
    Engine.SpriteList.AddSprite(TEnemyWeapon1.Create(Self, X+10, Y+65, 0.6));
    Engine.SpriteList.AddSprite(TEnemyWeapon1.Create(Self, X+85, Y+65, 0.6));
    AnimNr:=3;
  end;
end;

procedure TEnemy2.Hit(aSprite: TSprite);
begin
  Engine.SpriteList.AddSprite(TBang.Create(Self, CenterX, CenterY));
  Dead;
end;

function TEnemy2.AllowHit(aSprite: TSprite): Boolean;
begin
  Result:=(aSprite is TPlayerWeapon) or (aSprite is TPlayer);
end;

{ TEnemy3 }

constructor TEnemy3.Create(aParent: TSprite; aX, aY, aSpeed: Single);
begin
  inherited Create(aParent, 'Enemy1', 4, aX, aY, aSpeed);
  AnimDefaultNr:=1;
  FireTimer:=TTimer.Create(300);
end;

procedure TEnemy3.Move(aMoveCount: Single; aKeys: TKeys; aMouse: TMouse);
begin
  inherited;
  X:=X-Speed*aMoveCount;
  if X<-AnimBmp.Width then
    Dead;

  if FireTimer.Fire(aMoveCount) then begin
    Engine.SpriteList.AddSprite(TEnemyWeapon1.Create(Self, X+10, Y+65, 0.6));
    Engine.SpriteList.AddSprite(TEnemyWeapon1.Create(Self, X+85, Y+65, 0.6));
    AnimNr:=3;
  end;
end;

procedure TEnemy3.Hit(aSprite: TSprite);
begin
  Engine.SpriteList.AddSprite(TBang.Create(Self, CenterX, CenterY));
  Dead;
end;

function TEnemy3.AllowHit(aSprite: TSprite): Boolean;
begin
  Result:=(aSprite is TPlayerWeapon)  or (aSprite is TPlayer);
end;

{ TEnemy4 }

constructor TEnemy4.Create(aParent: TSprite; aX, aY, aSpeed: Single);
begin
  inherited Create(aParent, 'Enemy2', 4, aX, aY, aSpeed);
  AnimDefaultNr:=1;
  FireTimer:=TTimer.Create(150);
end;

procedure TEnemy4.Move(aMoveCount: Single; aKeys: TKeys; aMouse: TMouse);
begin
  inherited;
  X:=X+Speed*aMoveCount;
  Y:=-(X-300)*(X-300)/300+400;

  if (X>SCREEN_WIDTH) or (Y<-500) then
    Dead;

  if FireTimer.Fire(aMoveCount) then begin
    Engine.SpriteList.AddSprite(TEnemyWeapon2.Create(Self, X+56, Y+58, 0.6));
    AnimNr:=3;
  end;
end;

procedure TEnemy4.Hit(aSprite: TSprite);
begin
  Engine.SpriteList.AddSprite(TBang.Create(Self, CenterX, CenterY));
  Dead;
end;

function TEnemy4.AllowHit(aSprite: TSprite): Boolean;
begin
  Result:=(aSprite is TPlayerWeapon) or (aSprite is TPlayer);
end;

end.
