{*******************************************************************************
 Project       : "Space Shooter ..." ?
 Date          : 2007-07-11
 Version       : 1.3
 Compiler Ver. : Delphi 2007
 Author        : Denny Koberling
 E-Mail        : JasonVoorhees2k@arcor.de
 Copyright     : Copyright (c) 2007 Denny Koberling
*******************************************************************************}

unit UType;

interface

uses Windows, SysUtils, UAnimBmp;

var
  SCREEN_WIDTH: Integer;
  SCREEN_HEIGHT: Integer;

CONST
  MAXFPS = 120;

  MAX_STARS = 200;

  cBlack = $0000;
  cWhite = $FFFF;

type
  TKeys = array[0..255] of Boolean;

  TMouse = record
             Pos: TPoint;
             LButton, RButton: Boolean;
           end;

  TLayer = 0..9;

  {Layer 
   0 = HUD
   1 =
   2 =
   3 = Effect
   4 = Player & Enemy
   5 = Weapons
   6 =
   7 =
   8 = Stars
   9 = Background}

  function GetBitmap(aFileName: String): TAnimBmp;
  function RectInRect(aR1, aR2: TRect): Boolean;

var
  Path: String;

implementation

uses Classes;

var
  ObjList: array of TAnimBmp;

function GetBitmap(aFileName: String): TAnimBmp;
var
  i: Integer;
begin
  Result:=nil;
  for i:=Low(ObjList) to High(ObjList) do
    if AnsiUpperCase(ObjList[i].Name)=AnsiUpperCase(aFileName) then begin
      Result:=ObjList[i];
      Break;
    end;
end;

function RectInRect(aR1, aR2: TRect): Boolean;
begin
  Result:=((aR1.Left>=aR2.Left) and (aR1.Left<=aR2.Right) or
           (aR1.Right>=aR2.Left) and (aR1.Right<=aR2.Right)) and
          ((aR1.Top>=aR2.Top) and (aR1.Top<=aR2.Bottom) or
           (aR1.Bottom>=aR2.Top) and (aR1.Bottom<=aR2.Bottom));
end;

procedure ListFileDir(aPathDir: String; aFileList: TStrings);
var
  SR: TSearchRec;
begin
  if FindFirst(aPathDir, faAnyFile, SR) = 0 then begin
    repeat
      if SR.Attr<>faDirectory then
        aFileList.Add(SR.Name);
    until FindNext(SR) <> 0;
    FindClose(SR);
  end;
end;

procedure ObjCreate;
var
  i: Integer;
  sl: TStringList;
begin
  sl:=TStringList.Create;
  try
    ListFileDir(Path+'Graphics\*.obj', sl);
    SetLength(ObjList, sl.Count);
    for i:=Low(ObjList) to High(ObjList) do
      ObjList[i]:=TAnimBmp.Create(Path+'Graphics\'+sl[i]);
  finally
    sl.Free;
  end;
end;

procedure ObjDestroy;
var
  i: Integer;
begin
  for i:=Low(ObjList) to High(ObjList) do
    FreeAndNil(ObjList[i]);
end;

initialization
  ObjCreate;

finalization
  ObjDestroy;

end.
